/****************************************************************************/
/*                                                                        	*/
/*            Copyright (c) 1999-2000 by Atmel Corporation		          	*/
/*                                                                        	*/
/*  This software is copyrighted by and is the sole property of Atmel	  	*/
/*  Corporation.  All rights, title, ownership, or other interests		  	*/
/*  in the software remain the property of Atmel Corporation.  This       	*/
/*  software may only be used in accordance with the corresponding			*/ 
/*  license agreement.  Any un-authorized use, duplication, transmission,	*/
/*  distribution, or disclosure of this software is expressly forbidden.  	*/
/*                                                                        	*/
/*  This Copyright notice may not be removed or modified without prior		*/
/*  written consent of Atmel Corporation.                                 	*/
/*                                                                        	*/
/*  Atmel Corporation, Inc. reserves the right to modify this software    	*/
/*  without notice.                                                       	*/
/*                                                                        	*/
/*  Atmel Corporation.                                                    	*/
/*  2325 Orchard Parkway               literature@atmel.com               	*/
/*  San Jose, CA 95131                 http://www.atmel.com               	*/
/*                                                                        	*/
/****************************************************************************/


/****************************************************************************/
/****************************************************************************/
/**                                                                       	*/
/** FastVNET (PCMCIA) NDIS Miniport Driver                                	*/
/**                                                                       	*/
/**   Application interface related definitions	                          	*/
/**                                                                       	*/
/****************************************************************************/
/****************************************************************************/



#ifndef __appint_h_OK__
#define __appint_h_OK__

#include "vnet.h"

#ifndef UNDER_CE
#ifdef WINDOWS_OS
#pragma pack (push,1)
#endif
#endif

#define SHORT_WEP_KEY_SIZE				5
#define LONG_WEP_KEY_SIZE				13

#define WEP_DISABLED					0
#define WEP_64bit						1
#define WEP_128bit						2

///////////////////////////////////////////////////////////////////////////
//	SUPPORT_APP_INFO 
///////////////////////////////////////////////////////////////////////////

//
//	This structure is exchanged betwwen the driver and the 
//	monitor application. 
//


typedef struct __DEVICE_CONFIGURATION
{
	UCHAR			OperatingMode;					//AD_HOC_MODE or INFRASTRUCTURE_MODE
	UCHAR			Channel;						
	UCHAR			SSID[MAX_SSID_LENGTH];			
	UCHAR			SSIDlength;
	UCHAR			TxRate;							//0-4
	
	UCHAR			PowerMgmtMode;					// PM_MODE_ACTIVE or PM_MODE_POWER_SAVE

	UCHAR			PreambleType;					// LONG_PREAMBLE or SHORT_PREAMBLE
	
	USHORT			FragmentationThreshold;			
	USHORT			RtsCtsThreshold;

	UCHAR			BSSID[6];

	UCHAR			StationState;
	UCHAR			Rssi;
	UCHAR			LinkQuality;
	ULONG			MgmtErrorCode;			


}DEVICE_CONFIGURATION, *PDEVICE_CONFIGURATION;



//
// VERSION_INFO exchanged between driver/application 
//
typedef struct __VERSION_INFO{
	UCHAR	DriverMajorVersion;
	UCHAR	DriverMinorVersion;
	UCHAR	DriverSubVersion;
	USHORT	DriverBuild;

	USHORT	FwMajorVersion;
	USHORT	FwMinorVersion;
	USHORT	FwSubVersion;
	USHORT	FwBuild;

}VERSION_INFO;

//
// WEP_INFO is exchanged between driver/application 
//


typedef struct __WEP_INFO
{
	UCHAR						WepKeyToUse;
	UCHAR						WepMode;
	USHORT						AuthenticationType;
	UCHAR						EncryptionLevel;					//0:Disabled,1:64bit,2:128 bit
	UCHAR						WepKey1[LONG_WEP_KEY_SIZE];
	UCHAR						WepKey2[LONG_WEP_KEY_SIZE];
	UCHAR						WepKey3[LONG_WEP_KEY_SIZE];
	UCHAR						WepKey4[LONG_WEP_KEY_SIZE];
} WEP_INFO;


#define MAX_BSS_ENTRIES		12 
#define MAX_IBSS_ENTRIES	12

typedef struct __BSS_INFO{
	UCHAR	Channel;
	UCHAR	SSID[MAX_SSID_LENGTH];
	UCHAR	SSIDsize;
	UCHAR	BSSID[6];
	UCHAR	RSSI;
	UCHAR	UsingWEP;
	UCHAR	PreambleType;
	USHORT  BeaconPeriod;
	UCHAR	BSStype;
}BSS_INFO;

typedef struct __IBSS_INFO{
	UCHAR	MACaddress[6];
}IBSS_INFO;


typedef struct __STATISTICS{
	ULONG	TxDataPacketsOk;
	ULONG	TxDataPacketsError;
	ULONG	TxMgmtPacketsOk;
	ULONG	TxMgmtPacketsError;

	ULONG	RxDataPacketsOk;
	ULONG	RxDataPacketsError;
	ULONG	RxMgmtPacketsOk;
	ULONG	RxMgmtPacketsError;

	ULONG	RxLost;
	ULONG	TxPacketsRejectedNotReady;
	ULONG	TxPacketsRejectedResources;
	ULONG	MatchingBeacons;
	

}STATISTICS;





#define SITE_SURVEY_IDLE				0
#define SITE_SURVEY_IN_PROGRESS			1
#define SITE_SURVEY_COMPLETED			2


#ifndef UNDER_CE
#ifdef WINDOWS_OS
#pragma pack (pop)
#endif
#endif

#endif
