#!/bin/bash
# ATMEL Wireless Card Setup for linux
# Copyright 2001 (c) ATMEL Hellas

CONFIG_MK=config.mk
CONFIG_H=config.h

function Checkxforms ()
{
	if [ -r /usr/include/X11/Xlib.h -o -r /usr/X11R6/include/X11/Xlib.h ]; then
		for f in /usr/{X11/,X11R6/,local/,}lib/libforms.{so,a} ; do
			if [ -r $f ]; then break; fi
		done
		for g in /usr/{X11/,X11R6/,local/,}include/{,,X11/}forms.h ; do
			if [ -r $g ]; then break; fi
		done
		if [ -r $f -a -r $g ] ; then
			HAS_FORMS=y
		else
			cd misc
			bunzip2 forms.tar.bz2 &>/dev/null
			tar xvf forms.tar &>/dev/null
			bzip2 forms.tar &>/dev/null
			cd xforms
			make -s
			cd ..
			cd ..
			HAS_FORMS=y
		fi
	else
		echo "X Windows include files missing"
	fi
}

function CheckKernelVersion
{
    if [ -d $1 ]; then
        if [ -f $1/Makefile ]; then
            for TAG in VERSION PATCHLEVEL SUBLEVEL EXTRAVERSION ; do
                eval `sed -ne "/^$TAG/s/[ 	]//gp" $1/Makefile`
            done
            FV="$VERSION.$PATCHLEVEL.$SUBLEVEL$EXTRAVERSION"
            if [ "$LV" = "$FV" ]; then
                if [ "$PATH_FOUND" = "n" ]; then
                    PATH_FOUND=y
                    SRCPATH=$1
                fi
            fi
        fi
    fi
}



SRCPATH=null
PCMCIA_SRC=null
PATH_FOUND=n
HAS_FORMS=n
MAN_PATH=$(man -w ls|xargs dirname)
MPA=/usr/src
CONFIGP="/etc/pcmcia"
USB_CONFIG="/etc/hotplug"
LV=$(uname -r)
USBDES="/lib/modules/$LV/kernel/drivers/usb"
if [ ! -d $USBDES ]; then
	mkdir $USBDES
fi
PCMCIADES="/lib/modules/$LV/pcmcia"

MODPROBE_PATH=$(which modprobe)

rm -f $CONFIG_MK
rm -f $CONFIG_H
Checkxforms

if [ -L /lib/modules/$LV/build ]&&[ -f /lib/modules/$LV/build/Makefile ]; then
    SRCPATH=/lib/modules/$LV/build
else
    for f in $MPA/*; do CheckKernelVersion $f ; done
fi

echo "Kernel Version Running $LV"

if [ "$SRCPATH" = "null" ]; then
    echo -e "Kernel Source Directory Not Found. Please give path : \c"
    read SRCPATH
    if [ ! -d $SRCPATH ]; then
        echo "Invalid Path"
        rm -f config.mk
    fi
else
    echo Found Kernel Source Directory "($SRCPATH)"
fi

if [ $# -gt 0 ]; then
    PCMCIA_IN_KERNEL=$(grep -c CONFIG_PCMCIA= $SRCPATH/.config)

    if [ $PCMCIA_IN_KERNEL -eq 1 ] ; then
	PCMCIA_SRC=$SRCPATH
    else
	for f in $SRCPATH/pcmcia*; do
	    if [ -L $f ]; then
		if [ -d $f/include ]; then
		    PCMCIA_SRC=$f
		    break
	    fi
	    fi

	    if [ -d $f ]; then
	    if [ -d $f/include ]; then
		    PCMCIA_SRC=$f
		    break
	    fi
	    fi
	done
	if [ "$PCMCIA_SRC" = "null" ]; then
	    echo "Pcmcia card services source needed but not found"
	    echo "Please give desired path (or just press enter to"
        echo "use the kernel source header's if PCMCIA support"
        echo "is inside the kernel) #"
	    read PCMCIA_SRC
	fi
	if [ ! -d $PCMCIA_SRC/include ] && [ "$PCMCIA_SRC" != "" ]; then
	    echo "Invalid directory. Check pcmcia source path and retry"
	    exit -1;
	fi
    fi
fi

echo LINUX = $SRCPATH >> $CONFIG_MK
echo HAS_FORMS = $HAS_FORMS >> $CONFIG_MK
if [ $# -gt 0 ]; then
    echo PCMCIA_DES = $PCMCIADES >> $CONFIG_MK
    echo PCMCIA_CONFIG = $CONFIGP >> $CONFIG_MK
    echo PCMCIA_SRC = $PCMCIA_SRC >> $CONFIG_MK
fi
echo USB_DES = $USBDES >> $CONFIG_MK
echo USB_CONFIG = $USB_CONFIG >> $CONFIG_MK
echo MAN_PATH = $MAN_PATH >> $CONFIG_MK
echo >> $CONFIG_MK
echo >> $CONFIG_MK
