#include <linux/module.h>
#include "vnetusba.h"


extern struct usb_driver vnet_driver;

int VNetAllocMem(void **Ptr, USHORT size)
{
	if(in_interrupt()){
		*Ptr = kmalloc(size, GFP_ATOMIC);
	}else{
		*Ptr = kmalloc(size, GFP_KERNEL);
	}
	if(*Ptr==NULL){
		return -1;
	}
	return 0;		
}

void VNetFree(PUCHAR Ptr)
{
	kfree(Ptr);
}

void VnetTimer(PVNet_ADAPTER Adapter, UCHAR timeout)
{
	if(timeout==0){
		del_timer(Adapter->CtrlTimer);
		return;
	}
	Adapter->flags |= TimerReScheduled;
	
	if(timer_pending(Adapter->CtrlTimer)!=0) {
		mod_timer( Adapter->CtrlTimer, (jiffies+(timeout*HZ)/10) );
	} else {
		Adapter->CtrlTimer->expires = jiffies + (timeout*HZ)/10 ;
		add_timer(Adapter->CtrlTimer);
	}
}

void VNetFreeSkb(struct sk_buff *skb)
{
	if(in_interrupt())
		dev_kfree_skb_irq(skb);
	else
		dev_kfree_skb(skb);
}

int rx_kernel_part(PVNet_ADAPTER Adapter, struct sk_buff **skb, 
				USHORT PacketLength, UCHAR DataOffset)
{
	*skb = dev_alloc_skb(PacketLength+2);

	if(!(*skb))
		return -1;
	
	skb_reserve(*skb, 2);
	memcpy(skb_put(*skb, PacketLength), 
		(PUCHAR)Adapter->RxBuffUp.WirelessPacket+DataOffset, 
		PacketLength);
	return 0;	
}

VnetUsbReset(PVNet_ADAPTER Adapter)
{
	struct usb_interface *iface = &Adapter->config->interface[0];
	  
	usb_driver_release_interface(&vnet_driver, iface);

	info("Reseting Usb Device\n");
	usb_inc_dev_use(Adapter->usb);
	usb_reset_device(Adapter->usb);
	usb_dec_dev_use(Adapter->usb);
	info("Reset completed.");
	
	if(usb_get_configuration(Adapter->usb)!=0){
	    info("Get configuration failed");
	    return -1;
	}
	
	if(usb_set_configuration(Adapter->usb, 1)!=0){
		info("Set configuration failed");
		return -1;
	}
	
	Adapter->usb->actconfig = Adapter->usb->config;
	Adapter->config = Adapter->usb->actconfig;
	iface = Adapter->config->interface;
	usb_driver_claim_interface(&vnet_driver, iface, Adapter);
	return 0;
}

int VNetSchedule(PVNet_ADAPTER Adapter, ULONG flag)
{
	DECLARE_WAITQUEUE(wait, current);
	if(in_interrupt()){
		info("scheduling in interrupt?!!!");
		return -1;
	}
	dbgusbnl("%08x-%08x-%08x", Adapter->ctrl_wait, &wait, Adapter)
	add_wait_queue(Adapter->ctrl_wait, &wait);
	set_current_state(TASK_INTERRUPTIBLE);
	Adapter->flags |= flag;
	dbgusbnl("Zooop")
	schedule();
	set_current_state(TASK_RUNNING);
 	remove_wait_queue(Adapter->ctrl_wait, &wait);
	return 0;
}

void VNetIf_queue(struct net_device *dev, UCHAR c)
{
	if(c){
		netif_wake_queue(dev);
		((PVNet_ADAPTER)dev->priv)->flags &= ~TX_STOPPED;
	}else{
		netif_stop_queue(dev);
		((PVNet_ADAPTER)dev->priv)->flags |= TX_STOPPED;	
	}
}

int InitAdapter(PVNet_ADAPTER Adapter)
{
	PTX_LIST	TxList;
	UCHAR		i, added=0;

	Adapter->CtrlTimer 	= (struct timer_list *)kmalloc(sizeof(struct timer_list), GFP_KERNEL);
	if(Adapter->CtrlTimer==NULL){
	  err("timer alloc failed");
		return -1;
	}
	Adapter->netstats 	= (struct net_device_stats *)kmalloc(sizeof(struct net_device_stats), GFP_KERNEL);
	if(Adapter->netstats==NULL){
	  err("netstats alloc failed");
		return -1;
	}
	Adapter->w_stats 	= (struct iw_statistics	*)kmalloc(sizeof(struct iw_statistics), GFP_KERNEL);
	if(Adapter->w_stats==NULL){
	  err("wstats alloc failed");
		return -1;
	}
	Adapter->devreq 	= (devrequest *)kmalloc(sizeof(devrequest), GFP_KERNEL);
	if(Adapter->devreq==NULL){
	  err("device request alloc failed");
		return -1;
	}
	Adapter->lock 		= (spinlock_t *)kmalloc(sizeof(spinlock_t), GFP_KERNEL);
	if(Adapter->lock==NULL){
	  err("lock alloc failed");
		return -1;
	}
	Adapter->ctrl_wait 	= (wait_queue_head_t *)kmalloc(sizeof(wait_queue_head_t), GFP_KERNEL);
	if(Adapter->ctrl_wait==NULL){
	  err("wait queue alloc failed");
		return -1;
	}
	Adapter->remove_wait = (wait_queue_head_t *)kmalloc(sizeof(wait_queue_head_t), GFP_KERNEL);	
	if(Adapter->remove_wait==NULL){
	  err("wait queue alloc failed");
		return -1;
	}
	Adapter->TxList 	= (struct list_head *)kmalloc(sizeof(struct list_head), GFP_KERNEL);	
	if(Adapter->TxList==NULL){
	  err("list alloc failed");
		return -1;
	}
	Adapter->freeTx 	= (struct list_head *)kmalloc(sizeof(struct list_head), GFP_KERNEL);
	if(Adapter->freeTx==NULL){
	  err("list alloc failed");
		return -1;
	}
		
	init_timer(Adapter->CtrlTimer);

	init_waitqueue_head(Adapter->ctrl_wait);
	init_waitqueue_head(Adapter->remove_wait);
	
//	SPIN_LOCK_UNLOCKED(Adapter->lock);

	INIT_LIST_HEAD(Adapter->TxList);//empty list for incoming packets...	
	INIT_LIST_HEAD(Adapter->freeTx);//pool of free tx entries.....		
	
	// Ctrl Timer used for (Re)Authentication, (Re)Association requests.....
	
	Adapter->CtrlTimer->expires		= jiffies+ (HZ);// one second.
	Adapter->CtrlTimer->function	= TimerExpired;
	Adapter->CtrlTimer->data		= (ULONG)Adapter;
	
	Adapter->rx_urb = usb_alloc_urb(0);
	Adapter->tx_urb = usb_alloc_urb(0);
	Adapter->ctrl_urb = usb_alloc_urb(0);
	if( (Adapter->rx_urb==NULL)||
		(Adapter->tx_urb==NULL)||
		(Adapter->ctrl_urb==NULL)){
	  err("usb_alloc_urb failed");
		return -1;
	}
	
	for(i=0; i<10; i++){
 		// allocate 10 small structures and add them to our free list....
		VNetAllocMem((void *)&TxList, sizeof(TX_LIST));
		if(TxList == NULL)
			continue;
		TxList->Adapter = Adapter;
		VNetAllocMem((void *)&TxList->tx_buff, sizeof(TX_BUFFER));
		if(TxList->tx_buff==NULL)
			continue;
		TxList->skb = NULL;
		list_add(&TxList->tx_list, Adapter->freeTx);
		added++;
	}
	
	if(!added){
		err("Unable to allocate enough memory");
		return -1;
	}
	
	SetParameters(Adapter);
	memset(Adapter->CmdStatus, 0, 40);
	memset((PUCHAR)&Adapter->Stats, 0, sizeof(STATISTICS));
	memset((PUCHAR)Adapter->netstats, 0, sizeof(struct net_device_stats));
	memset((PUCHAR)Adapter->w_stats, 0, sizeof(struct iw_statistics));
	return 0;
}
